<?php
class WPJAM_Comments_Admin{
	public static function __callStatic($method, $args){
		$post_type		= wpjam_get_plugin_page_setting('post_type');
		$comment_type	= wpjam_get_plugin_page_setting('comment_type');
		$object			= wpjam_get_comment_type_object($comment_type, $post_type);

		if(str_ends_with($method, '_by_type')){
			$post_id	= (int)wpjam_get_data_parameter('post_id');

			if(!$post_id){
				$query		= self::get_query();
				$posts		= $query->posts;
				$post_id	= $posts ? current($posts)->ID : 0;
			}

			if($post_id){
				$method		= wpjam_remove_postfix($method, '_by_type');
				$callback	= method_exists($object, $method) ? [$object, $method] : [$object->model, $method];

				if($method == 'query_data'){
					return call_user_func($callback, $post_id, ...$args);
				}else{
					return call_user_func($callback, ...$args);
				}
			}	
		}elseif($method == 'get_query'){
			static $query;

			if(!isset($query)){
				$left_paged	= wpjam_get_data_parameter('left_paged') ?: 1;
				$left_type	= wpjam_get_data_parameter('left_type');
				$left_type	= $left_type ?: ($object->post_meta ? 'most' : 'new');

				$query_args	= [
					'post_type'			=> $post_type,
					'paged'				=> $left_paged,
					'post_status'		=> '',
					'posts_per_page'	=> 10
				];

				if(in_array($left_type, ['publish', 'most'])){
					if($left_type == 'most'){
						if($object->name == 'comment'){
							$query_args['orderby']		= 'comment_count';
							$query_args['comment_count']= [
								'value'		=> 1,
								'compare'	=> '>='
							];
						}else{
							$query_args['orderby']	= 'meta_value_num';
							$query_args['meta_key']	= $object->plural;
						}
					}
				}elseif($left_type == 'new'){
					add_filter('posts_clauses', [self::class, 'filter_clauses']);
				}

				$query	= new WP_Query($query_args);
			}

			return $query;
		}elseif($method == 'filter_clauses'){
			$wpdb		= $GLOBALS['wpdb'];
			$type_str	= $comment_type	== 'comment' ? "'comment', ''" : "'".esc_sql($comment_type)."'";
			$ct_where	= "ct.comment_type IN ({$type_str}) AND ct.comment_parent=0 AND ct.comment_approved NOT IN ('spam', 'trash', 'post-trashed')";

			$clauses	= $args[0];

			$clauses['fields']	.= ', ct.comment_post_ID, MAX(ct.comment_ID) as cid';
			$clauses['join']	= "INNER JOIN {$wpdb->comments} AS ct ON {$wpdb->posts}.ID = ct.comment_post_ID AND {$ct_where}";
			$clauses['groupby']	= "ct.comment_post_ID";
			$clauses['orderby']	= "cid DESC";

			return $clauses;
		}elseif($method == 'col_left'){
			$query		= self::get_query();
			$posts		= $query->posts;
			$total		= $query->found_posts;

			$pt_label	= get_post_type_object($post_type)->label;
			$options	= ['new'=>'最新'.$object->label, 'most'=>'最多'.$object->label, 'publish'=>'最新'.$pt_label];

			if($object->post_meta){
				$options	= wpjam_array_except($options, 'new');
			}
			?>
			<div class="tablenav">
				<div class="alignleft actions"><?php 
				echo wpjam_render_field([
					'key'		=> 'left_type',
					'type'		=> 'select',
					'class'		=> 'left-filter',
					'value'		=> wpjam_get_data_parameter('left_type'),
					'options'	=> $options
				]);
				?></div>
			</div>
			<table class="widefat striped">
				<thead>
					<tr><th><?php echo $pt_label; ?></th></tr>
				</thead>
				<tbody>
				<?php
				$post_id	= 0;

				if($posts){

				$post_id	= wpjam_get_data_parameter('post_id');

				if($post_id){
					$post_ids	= array_column($posts, 'ID');

					if(!in_array($post_id, $post_ids)){ 
						array_unshift($posts, get_post($post_id));
					}
				}else{
					$post_id	= $posts[0]->ID;
				}

				$post_ids	= array_column($posts, 'ID');
				$counts		= [];

				foreach(wpjam_get_comment_counts($post_ids, ['type'=>$comment_type]) as $count){
					$comment_post_id	= $count['comment_post_ID'];
					$comment_approved	= $count['comment_approved'];

					$counts[$comment_post_id][$comment_approved]	= $count['count'];
				}
				?>
				<?php foreach($posts as $post){ 
					$class	= 'left-item';

					if($post_id == $post->ID){
						$class	.= ' left-current';
					}
				?>
					<tr data-id="<?php echo $post->ID; ?>" id="left-<?php echo $post->ID; ?>" class="<?php echo $class; ?>">
						<td><?php 

						$count		= $object->get_count($post->ID);

						$approved	= $counts[$post->ID][1] ?? 0;
						$unapproved	= $counts[$post->ID][0] ?? 0;

						if($approved != $count){
							$diff	= '不相同('.$approved.')';
						}else{
							$diff	= '';
						}

						if($object->name == 'comment'){
							$count	= $count ? $count.'条评论': (comments_open($post) ? '暂无评论' : '评论未开启');
						}else{
							// if($diff){
							// 	$count	= $approved;
							// 	update_post_meta($post->ID, $object->plural, $count);
							// }

							$count	= ($count ? $count.' ' : '暂无').$object->label;
						}

						// $count	.= $diff;

						$post_card	= '<p class="row-title column-response">';
						$post_card	.= '<span class="post-title">'.get_the_title($post).'</span>'._post_states($post, false);

						// if($unapproved){
						// 	$post_card	.= '<span class=""><span class="post-com-count post-com-count-pending"><span class="comment-count-pending" aria-hidden="true">'.$unapproved.'</span><span class="screen-reader-text">'.$unapproved.'条待审</span></span></span>';
						// }

						$post_card	.= '</p>';
						$post_card	.= '<p>';
						$post_card	.= '<span class="post-time">'.get_the_time('Y-m-d H:i:s', $post).'</span>';
						$post_card	.= '<span class="comment-count wp-ui-highlight">'.$count.'</span>';
						$post_card	.= '</p>';

						echo $post_card;

						?></td>
					</tr>
				<?php } }else{ ?>
					<tr class="no-items"><td class="colspanchange">找不到<?php echo $pt_label;?>。</td></tr>
				<?php } ?>
				</tbody>
				<tfoot>
					<tr><th><?php echo $pt_label; ?></th></tr>
				</tfoot>
			</table>

			<?php

			return ['total_items'=>$total, 'per_page'=>10, 'left_value'=>$post_id];
		}
	}

	public static function get($id){
		$data	= get_comment($id, ARRAY_A);
		$data	= wpjam_array_except($data, ['populated_children', 'post_fields']);

		if($data){
			foreach($data as $key => $value){
				if(str_starts_with($key, 'comment_')){
					$new_key	= wpjam_remove_prefix($key, 'comment_');
					$lower_key	= strtolower($new_key);

					$data[$new_key]	= $value;

					if($new_key != $lower_key){
						$data[$lower_key]	= $value;
					}
				}
			}

			$data['content']	= wp_strip_all_tags($data['content']);
			$data['meta']		= get_comment_meta($id) ?: [];
			$data['meta']		= array_map(function($item){ return maybe_unserialize($item[0]); }, $data['meta']);
		}

		return $data;
	}

	public static function query_data($args){
		foreach(['orderby', 'order'] as $key){
			$value	= wpjam_get_data_parameter($key, ['sanitize_callback'=>'sanitize_key']);

			if($value){
				$args[$key]	= $value;
			}
		}

		return self::query_data_by_type($args) ?: ['items'=>[], 'total'=>0];
	}

	public static function render_item($item){
		return self::render_item_by_type($item) ?: $item;
	}

	public static function get_fields($action_key='', $id=0){
		$fields	= self::get_fields_by_type($action_key, $id) ?: [];

		if($action_key == ''){
			$fields	= array_merge(['author'=> ['title'=>'用户',	'type'=>'text',	'show_admin_column'=>'only']], $fields);
		}

		return $fields;
	}

	public static function get_actions(){
		return self::get_actions_by_type() ?: [];
	}

	public static function value_callback($key, $id){
		return wpjam_get_metadata('comment', $id, $key);
	}
}

$comment_type	= wpjam_get_plugin_page_setting('comment_type');
$post_type		= wpjam_get_plugin_page_setting('post_type');
$object			= wpjam_get_comment_type_object($comment_type, $post_type);
$post_id		= (int)wpjam_get_data_parameter('post_id');

if($post_id){
	$result	= wpjam_validate_post($post_id, $post_type);

	if(is_wp_error($result)){
		wp_die($result);
	}
}

do_action('wpjam_comments_page_load', $comment_type, $post_type);

wpjam_register_list_table($GLOBALS['plugin_page'], [
	'primary_key'	=> $object->post_meta ? '' : 'id',
	'plural'		=> $object->plural,
	'singular'		=> $comment_type,
	'model'			=> 'WPJAM_Comments_Admin',
	'per_page'		=> $object->post_meta ? 9999 : 20,
	'layout'		=> 'left',
	'left_key'		=> 'post_id'
]);

if(!$object->post_meta){
	wpjam_register_list_table_column('comment_date', ['title'=>'提交于', 'order'=>1]);
}

wp_add_inline_style('list-tables', "\n".join("\n", [
	'table.wp-list-table th,table.wp-list-table td {
		box-shadow: inset 0 -1px 0 rgba(0, 0, 0, 0.1);
	}

	table.wp-list-table tr:last-child th,
	table.wp-list-table tr:last-child td {
		box-shadow: none;
	}

	table.wp-list-table tr.unapproved + tr.approved th,
	table.wp-list-table tr.unapproved + tr.approved td {
		border-top: 1px solid rgba(0, 0, 0, 0.03);
	}

	table.wp-list-table .unapproved th,
	table.wp-list-table .unapproved td{background-color: #fcf9e8;}
	table.wp-list-table .unapproved th.check-column {border-left: 4px solid #d63638;}
	table.wp-list-table .read th.check-column{border-left: 4px solid #0073aa;}
	table.wp-list-table .unapproved th.check-column input,
	table.wp-list-table .read th.check-column input{margin-left: 4px;}
	table.wp-list-table .approve a,
	table.wp-list-table .mark_as_read a{color: #007017;}
	table.wp-list-table .unapprove a{color: #996800;}',

	'th.manage-column{min-width:28px;}',
	'th.column-author{width: 30%;}',
	'th.column-comment_date{width: 12%;}',
	'.widefat td.column-comment p{margin:0 0 .6em 0;}',
	'span.author-pad{float:left; margin-right:10px;}',

	'@media screen and (min-width: 782px){ #col-left{width: 30%;} #col-right{width: 70%} }',

	'div#col-left td{padding-left: 14px; padding-right: 20px;}',
	'div#col-left td a:focus{box-shadow: inherit;}',
	'div#col-left td span.post-time{font-size:smaller;}',
	'div#col-left td span.comment-count{font-size:smaller; float:right; padding:2px 4px;}'
])."\n");

add_action('admin_head', function(){
	?>
	<script type="text/javascript">
	jQuery(function($){
		$('body').on('click', 'a.reply_to', function(event){
			let parent	= $(this).data('parent');
			let top		= $('#comment-'+parent).offset().top-40;

			$('html').animate({scrollTop: top}, 500, function(){
				$('#comment-'+parent).animate({opacity: 0.1}, 500).animate({opacity: 1}, 500);
			});

		    event.preventDefault();
		});
	});
	</script>
	<?php
});